require_dependency "rechn_mgm_user_area/application_controller"

module RechnMgmUserArea
    class OrganisationsController < AuthenticatedController
        layout 'application'

        before_action :set_user, only: [:resend_invitation]
        before_action :set_membership, only: [:index, :create_invitation]

        def index
            @my_organisation = current_user.current_organisation
            @member_ships = nil
            if current_user.current_membership.is_primary?
                @member_ships = current_user.member_ships
            end
            @users = current_user.users_of_current_organisation
            @invited_users = current_user.invited_by_organisation
        end


        def new_invitation
            respond_to do |format|
                format.html
                format.js
            end
        end

        def create_invitation
            invited_user = DeviseAuthentication::User.where(email: organisation_params[:email]).first

            if invited_user.present?
                if invited_user.id == current_user.id
                    flash[:error] = I18n.t "You cannot add yourself!"
                    redirect_to user_root_path
                    return
                end

                if current_user.user_ids_of_current_organisation.include? invited_user.id
                    flash[:error] = I18n.t "This user is already in your organisation!"
                    redirect_to user_root_path
                    return
                end

                if current_user.invited_by_me.map{|u| u.id}.include? invited_user.id
                    flash[:error] = I18n.t "You already invited this user!"
                    redirect_to user_root_path
                    return
                end

                # User is already known: Add him directly
                success = current_user.add_user_to_current_organisation(invited_user.id)
                if success
                    flash[:info] = I18n.t "The invited user was added"
                else
                    flash[:error] = I18n.t "The invited user could not be added. Please contact administrator!"
                end
            else
                # User is not known: Invite him
                flash[:info] = I18n.t("invitation_sent", email: organisation_params[:email])
                DeviseAuthentication::User.invite!({email: organisation_params[:email], forename: organisation_params[:first_name], lastname: organisation_params[:last_name], invited_for_organisation_id: current_user.current_organisation.id}, current_user)
            end
            redirect_to user_root_path
        end

        def resend_invitation
            flash[:info] = I18n.t("invitation_sent", email: @user.email)
            DeviseAuthentication::User.invite!({email: @user.email, forename: @user.forename, lastname: @user.lastname}, current_user)

            redirect_to user_root_path
        end

        def switch_organisation
            session[:selected_organisation] = switch_organisation_params[:selected_organisation]
            render :json => { :error => "None", "ErrorCode" => 0 }
        end

        private
        def organisation_params
            params.require(:invitation).permit(:email, :first_name, :last_name)
        end

        def switch_organisation_params
            params.permit(:selected_organisation)
        end

        def set_user
            @user = DeviseAuthentication::User.find(params[:user_id])
        end

        def set_membership
            current_user.current_membership_id = session[:selected_organisation] if session[:selected_organisation].present?
        end
    end
end
