require_dependency "rechn_mgm_user_area/application_controller"

# TODO: Think whether move the implementation specific part (roles, favorites) to rechn_mgm_web? (se)
module RechnMgmUserArea
    class UsersController < AuthenticatedController
        layout 'application'
        protect_from_forgery except: [:set_favorite, :update_frontend_url], prepend: true #, with: :exception, unless: -> { request.format.json? }
        before_action :set_user, only: [:show, :edit, :update, :change_role_dialog, :change_role, :delete_invitation]
        before_action :set_current_membership, only: [:change_role_dialog, :change_role, :delete_user_membership]

        def index
            @users = current_user.users_of_current_organisation
            respond_to do |format|
                format.html do
                    redirect_to organisation_path
                end
                format.json do
                    # intentionelly left blank (user default render)
                end
            end

        end

        # GET /users/1/edit
        def edit
        end

        def update
            if @user.update(user_params)
                #redirect_to @document, notice: 'Document was successfully created.'
                render :edit
            else
                flash[:error] = "The data could not be stored"
                render :edit
            end
        end

        def memberships
            @member_ships = current_user.member_ships.no_admin.hide_own(current_user.hide_own_organisation)
        end

        def show
        end

        def show_me
            @user = current_user
            render :show
        end

        def favorites
            @user = current_user
        end

        def set_favorite
            @error   = 0
            @message = "success"
            if current_user.blank?
                @error = 1
                @message = "User could not be found in database."
            end

            if @error == 0
                selected_favorite = current_user.favorites.select { |f| f["_id"] == direct_params["id"] }.first
                fav_hash = direct_params[:favorites].to_h
                fav_hash[:_id] = direct_params[:id]
                if selected_favorite.present?
                    # Overwrite
                    selected_favorite = fav_hash
                else
                    # New
                    current_user.favorites.push fav_hash
                    if !current_user.save
                        @error = 2
                        @message = "Unable to save: #{current_user.errors[:messages]}"
                    end
                end
            end
        end

        def remove_favorite
            @error   = 0
            @message = "success"
            index_of_favorite = current_user.favorites.index { |f| f["_id"] == direct_params["id"] }
            current_user.favorites.delete_at(index_of_favorite)
            if !current_user.save
                @error = 2
                @message = "Unable to save: #{current_user.errors[:messages]}"
            end
        end

        def destroy
            current_user.destroy
            redirect_to "/logout"
        end

        def delete_user_membership
            success = current_user.delete_user_from_my_organisation(assignment_params[:user_id])
            if success
                flash[:info] = I18n.t "Your member was removed"
            else
                flash[:error] = I18n.t "Your member could not be removed"
            end

            redirect_to user_root_path
        end

        def delete_invitation
            success = @user.delete_invitation
            if success
                flash[:info] = I18n.t "Your invitee was removed"
            else
                flash[:error] = I18n.t "Your invitee could not be removed"
            end

            redirect_to user_root_path
        end

        def delete_membership_to_organisation
            # Switch to primary organisation first.
            session[:selected_organisation] = nil

            success = current_user.delete_membership_to_organisation(assignment_params[:membership_id])
            if success
                flash[:info] = I18n.t "Your membership was removed"
            else
                flash[:error] = I18n.t "Your membership could not be removed"
            end

            redirect_to user_root_path
        end

        def change_role_dialog
            @user.current_membership_id = @current_membership.id

            respond_to do |format|
                format.html
                format.js
            end
        end

        def change_role
            @user.current_membership_id = @current_membership.id

            new_params = change_role_params
            if (@user.id == current_user.id) && !new_params[:roles].include?("admin")
                # I cannot remove the admin role from myself.
                new_params[:roles].push "admin"
            end

            success = @user.update(new_params)
            if success
                flash[:info] = "The role of User #{@user.forename} #{@user.lastname} was changed."
            else
                flash[:error] = "The role of User #{@user.forename} #{@user.lastname} could not be changed."
            end


            redirect_to user_root_path
        end

        def update_frontend_url
            request_body = JSON.parse(request.body.read)
            session[:frontend_url] = request_body["frontend_url"]

            render json: {}, status: :ok
        end

        private
        # Use callbacks to share common setup or constraints between actions.
        def set_user
            @user = DeviseAuthentication::User.find(params[:id])
        end

        def set_current_membership
            @user = current_user if @user.nil?
            current_user.current_membership_id = session[:selected_organisation] if session[:selected_organisation].present?
            current_organisation = RechnMgmUserArea::MemberShip.find(current_user.current_membership_id).organisation
            @current_membership = RechnMgmUserArea::MemberShip.where(user_id: @user.id, organisation_id: current_organisation.id).first
        end

        # Only allow a trusted parameter "white list" through.
        def direct_params
            params.permit(:id, :favorites => {})
        end

        def user_params
            params.require(:user).permit(:id, :email, :forename, :lastname, :company, :street, :street_no, :zipcode, :city, :return_address)
        end

        def assignment_params
            params.permit(:user_id, :membership_id)
        end

        def change_role_params
            params.require(:user).permit(:roles=>[]).to_h
        end
    end
end
