# Decorator to add some special features to a User
# TODO: This is a strong relationship to DeviseAuthentication and will to stop work if we use anything different!! (se)
# TODO: Think whether move the implementation specific part (roles, favorites) to rechn_mgm_web? (se)

DeviseAuthentication::User.class_eval do
    # Hack to map module name. Otherwise has_many :organisations will fail..
    Organisation = RechnMgmUserArea::Organisation

    #.             1     2           4
    ROLES     = %w[admin buchhaltung sachbearbeiter]

    field :forename,        :type => String, :required => true
    field :lastname,        :type => String, :required => true
    field :company,         :type => String
    field :street,          :type => String
    field :street_no,       :type => String
    field :zipcode,         :type => String
    field :city,            :type => String
    field :return_address,  :type => String
    ## User Role
    field :favorites,       :type => Array,     :default => []

    attr_accessor :current_membership_id

    ## MemberShip relations
    has_many :member_ships,     :class_name => RechnMgmUserArea::MemberShip, :dependent => :destroy
    has_many :organisations,    :through => :member_ships

    ## Invitation meta data
    field    :invited_by_id,                :type => String
    field    :invitation_token,             :type => String
    field    :invited_for_organisation_id,  :type => String
    field    :hide_own_organisation,        :type => NoBrainer::Boolean, :default => false

    after_create    :account_created

    def user_name
        "#{self.forename} #{self.lastname}"
    end

    def is?(role)
        roles.include?(role.to_s)
    end

    def roles=(roles)
        self.roles_mask = (roles & ROLES).map { |r| 2**ROLES.index(r) }.inject(0, :+)
    end

    def roles
        ROLES.reject do |r|
            ((roles_mask.to_i || 0) & 2**ROLES.index(r)).zero?
        end
    end

    def current_membership_id=(value)
        assigned_membership = RechnMgmUserArea::MemberShip.find( value )
        fail "Cannot set membership with this id. It does not belong to me!" if assigned_membership.user_id != self.id
        @current_membership_id = assigned_membership.id
        @current_membership = assigned_membership
    end

    def current_membership
        fail if @current_membership_id.nil?
        @current_membership
    end

    def set_organisation(organisation)
        self.current_membership_id = RechnMgmUserArea::MemberShip.where(user_id: self.id, organisation_id: organisation.id).first.id
    end

    # TODO: Needs to be changed if a user can create multiple organisations
    def my_organisation
        self.member_ships.where(user_id: self.id, is_primary:true).first.organisation
    end

    def current_organisation
        init_current_membership

        @current_membership.organisation
    end

    def user_ids_of_current_organisation
        self.users_of_current_organisation.map{|u| u.id}
    end

    def users_of_current_organisation
        init_current_membership

        RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id).map do |m|
            u = DeviseAuthentication::User.find( m.user_id )
            u.current_membership_id = m.id
            u
        end
    end

    # Returns all memberships I'm admin of
    def memberships_as_admin
        init_current_membership

        RechnMgmUserArea::MemberShip.where(user_id: self.id).map{|m| m}.delete_if do |m|
            (m.roles_mask & 1) == 0
        end
    end

    def invited_by_me
        init_current_membership

        DeviseAuthentication::User.where(invited_by_id: self.id).map {|u| u}.delete_if do |u|
            # Check whether there is already an active membership ..
            u.invitation_token.blank? || (RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, user_id: u.id).count > 0)
        end
    end

    def invited_by_organisation
        init_current_membership

        DeviseAuthentication::User.where(:invited_by_id.not => nil, invited_for_organisation_id: @current_membership.organisation.id).map {|u| u}.delete_if do |u|
            # Check whether there is already an active membership ..
            u.invitation_token.blank? || (RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, user_id: u.id).count > 0)
        end
    end

    def delete_user_from_my_organisation(user_id)
        membership_to_remove = RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: self.current_organisation.id).first

        fail "I cannot remove myself" if membership_to_remove.is_primary
        membership_to_remove.destroy

        # Documents beloning to this user will be moved back to the origin
    end

    def delete_membership_to_organisation(organisation_id)
        RechnMgmUserArea::MemberShip.find(organisation_id).destroy

        # Documents beloning to this user will be moved back to the origin
    end

    def delete_invitation
        fail if self.invitation_token.blank?
        self.destroy
    end

    def add_user_to_current_organisation(user_id)
        my_organisation = self.current_organisation
        fail "User was already added to my organisation" if RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: my_organisation.id).count >= 1
        fail "Cannot add myself to my organistion" if user_id == self.id
        new_membership = RechnMgmUserArea::MemberShip.new(user_id: user_id, organisation_id: my_organisation.id, is_primary: false)

        new_membership.save
    end

    def invitation_state
        if self.invitation_token.present?
            :pending
        else
            :finished
        end
    end

    def users_of_my_organisation_with_role(role_index)
        init_current_membership

        RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, :roles_mask.ge => 2).map { |m| {id:m.user_id, roles_mask: m.roles_mask} }.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 2**role_index) }.map {|m| m[:id] }
    end

    # All Buchhalter of the current organisation
    def buchhaltung_user_ids
        users_of_my_organisation_with_role(ROLES.index("buchhaltung"))
    end

    # All Sachbearbeiter of the current organisation
    def sachbearbeiter_user_ids
        users_of_my_organisation_with_role(ROLES.index("sachbearbeiter"))
    end

    def all_buchhalter_user_ids_of_all_my_organisations
        all_my_organisations = RechnMgmUserArea::MemberShip.where(user_id: self.id).map { |m| m.organisation }
        all_memberships = []
        all_my_organisations.each{ |o| o.member_ships.each{|m| all_memberships.push m} }
        all_memberships.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 2) }.map { |m| m.user_id }
    end

    def is_owner_of_organisation(organisation_id)
        RechnMgmUserArea::MemberShip.where(user_id: self.id, organisation_id: organisation_id).first.is_primary
    end

    def roles_mask
        init_current_membership

        @current_membership.roles_mask
    end

    def roles_mask=(roles_mask)
        init_current_membership

        @current_membership.roles_mask = roles_mask
        @current_membership.save
    end

    def roles_mask_matches_role(roles_mask, role_to_match)
        return (roles_mask | role_to_match) == roles_mask
    end

    def init_current_membership
        if self.current_membership_id.blank?
            # Default to own Organisation if nothing else was specified.
            current_membership = RechnMgmUserArea::MemberShip.where( user_id: self.id, is_primary:true ).first
            if current_membership.present?
                self.current_membership_id = current_membership.id
            else
                Rails.logger.error "Unable to find the current membership. user: #{self.id}"
            end
        end
    end


    private
    def account_created
        # Every user has its own organisation.
        my_organisation = RechnMgmUserArea::Organisation.new(name: "#{self.forename} #{self.lastname}s Organisation")
        my_membership   = nil
        success = my_organisation.save

        # Add himmself to it.
        if success
            my_membership = RechnMgmUserArea::MemberShip.new(user_id: self.id, organisation_id: my_organisation.id, is_primary: true, roles_mask: 3)

            success = my_membership.save
        end

        # Rollback if something went wrong.
        unless success
            my_organisation.destroy
            my_membership.destroy
            Rails.logger.error("Unable to create organisation for user #{self.id}")
        end
    end

end
